/*
 * @(#)ChatServlet.java	1.12 96/11/03
 * 
 * Copyright (c) 1995 Sun Microsystems, Inc.  All Rights reserved Permission to
 * use, copy, modify, and distribute this software and its documentation for
 * NON-COMMERCIAL purposes and without fee is hereby granted provided that
 * this copyright notice appears in all copies. Please refer to the file
 * copyright.html for further important copyright and licensing information.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 * OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR
 * ITS DERIVATIVES.
 */

import java.io.*;
import java.util.*;
import java.servlet.*;
import sun.server.util.*;

/**
 * This is an example of a chat Servlet
 */
public class ChatServlet implements Servlet {
ServletStub stub;

    /**
     * Vector of client information
     */  
    Vector clientVector = new Vector();

    public void init(ServletStub stub) throws ServletException {
	this.stub = stub;
        //System.out.println("ChatServlet.init: enter");
    }

    public void service(ServletRequest req, ServletResponse res) 
	throws ServletException, IOException {
        //System.out.println("ChatServlet.service: enter");
        converse(req.getInputStream(), res.getOutputStream());
    }

    /**
     * when you get a new connection
     */
    void converse(InputStream clientInput, OutputStream clientOutput) {
        ClientInfo clientInfo;
	String line; 
        DataInputStream dis;
        boolean firstLine = true;

        clientInfo = new ClientInfo();
        clientInfo.clientOutput = new PrintStream(clientOutput);
        dis = new DataInputStream(clientInput);
	try {
	    while (true) {
	        line = dis.readLine();
                //System.out.println("line:");
                //System.out.println(line);
                if (line == null) {
                    break;
                }
                if (firstLine == true) { 
                    // the first line is the name
                    clientInfo.name = line;
                    firstLine = false;
	            addListener(clientInfo);
                } else {
                    // the subsequent lines are input to the chat room
		    addString(clientInfo.name + ": " + line);
		}
	    }
	} catch(Exception e) {
            log("ChatServerHandler.converse: " + e);
	}
	removeListener(clientInfo);
    }

    /**
     * add a listener to the chat room
     */
    synchronized void addListener(ClientInfo clientInfo) throws Exception {
        //System.out.println("ChatServlet.addListener: enter");
        log("ChatServlet.addListener: enter");
        log("ChatServlet.addListener: " + clientInfo.name + " entered");
        // let everyone else know
	addString(clientInfo.name + " has just entered the room.\n");
        clientVector.addElement(clientInfo);
        // let the current listener know who is in th room
	clientInfo.clientOutput.print("Welcome to Chat demo\n");
	clientInfo.clientOutput.flush();
    }

    /**
     * remove a listener from the chat room
     */
    synchronized void removeListener(ClientInfo clientInfo) {
        log("ChatServlet.removeListener: enter");
        log("ChatServlet.removeListener: " + clientInfo.name + " left");
        clientVector.removeElement(clientInfo);
	addString(clientInfo.name + " has left the room.\n");
    }

    /**
     * send a string to all the clients
     */
    synchronized void addString(String s) {
        Enumeration enum;
        ClientInfo clientInfo;

        enum = clientVector.elements();
	while(enum.hasMoreElements()) {
            clientInfo = (ClientInfo) enum.nextElement();
	    try {
	        clientInfo.clientOutput.print(s);
	        clientInfo.clientOutput.print("\n");
	        clientInfo.clientOutput.flush();
	    } catch(Exception e) {
	        removeListener(clientInfo);
	    }
        }
    }

    public String getServletInfo() {
        return "A chat servlet";
    }

    public void destroy() {
	log("destroy");
    }

    public void log(String message) {
	stub.getServletContext().log(this, message);
    }
}


/**
 * Information stored in the server about each client
 */
class ClientInfo {
    String name;
    PrintStream clientOutput;
}
