/*
 * @(#)FingerServlet.java	1.6 96/11/03 David Connelly
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.servlet.*;
import java.io.*;
import java.net.*;
import java.util.*;

import java.servlet.*;

/**
 * Finger servlet. This servlet uses the finger protocol to query
 * information about users on specified hosts. The query string
 * parameters <tt>user</tt>, <tt>hosts</tt>, and <tt>verbose</tt>
 * can be used to specify the user and hosts to query. The parameter
 * <tt>user</tt> is the user name, <tt>hosts</tt> is a comma-separated
 * list of host names to query, and <tt>verbose</tt> if specified will
 * cause verbose output to be generated. For example,
 * <pre>
 *     http:/goa/finger.html?user=dac&hosts=eno,doppio&verbose=yes
 * </pre>
 * This URL will request full information about user 'dac' on both
 * hosts 'eno' and 'doppio'.
 *
 * @version 	1.6, 11/03/96
 * @author 	David Connelly
 */
public
class FingerServlet extends GenericServlet {
    /*
     * Port number for finger daemon.
     */
    static final int FINGER_PORT = 79;

    /**
     * Handles a single finger request from the client.
     */
    public void service(ServletRequest req, ServletResponse res)
	throws ServletException, IOException
    {
	String user = req.getParameter("user");
	String hosts = req.getParameter("hosts");
	String verbose = req.getParameter("verbose");

	res.setContentType("text/html");

	PrintStream out = new PrintStream(res.getOutputStream());
	out.println("<html>");
	out.println("<head><title>Finger Servlet</title></head>");
	out.println("<body>");
	out.println("<h2>Finger results:</h2>");
	out.println("<pre>");
	if (hosts == null) {
	    finger(out, user, null, "yes".equalsIgnoreCase(verbose)) ;
	} else {
	    StringTokenizer st = new StringTokenizer(hosts, ",");
	    while (st.hasMoreTokens()) {
		String host = st.nextToken();
		out.println("[" + host + "]");
		try {
		    finger(out, user, host, "yes".equalsIgnoreCase(verbose));
		} catch (IOException e) {
		    out.println(e);
		}
		out.println();
	    }
	}
	out.println("</pre>");
	out.println("</body></html>");
    }

    /*
     * Sends finger output for a user and host to the specified output
     * stream.
     */
    void finger(OutputStream out, String user, String host, boolean verbose)
	throws IOException
    {
	// open connection to finger daemon
	Socket s;
	if (host == null) {
	    s = new Socket(InetAddress.getLocalHost(), FINGER_PORT);
	} else {
	    s = new Socket(host, FINGER_PORT);
	}
	// send finger command
	PrintStream ps = new PrintStream(s.getOutputStream());
	if (verbose) {
	    ps.print("/W ");
	}
	if (user != null) {
	    ps.print(user);
	}
	ps.print("\r\n");
	ps.flush();
	// copy results to output stream
	InputStream in = s.getInputStream();
	byte[] buf = new byte[512];
	int len;
	while ((len = in.read(buf, 0, buf.length)) != -1) {
	    out.write(buf, 0, len);
	}
	s.close();
    }
}
