/*
 * @(#)LogAnalyzerServlet.java	1.10 96/11/10 Jongyoon Lee
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.  All Rights reserved
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file copyright.html
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.servlet.*;
import java.servlet.http.*;
import java.io.*;
import java.util.*;

/**
 * LogAnalyzerServlet
 * 
 * This is the access_log version of log analyzer.
 * This servlet reads the access log of http server and
 * presents the access information.
 *
 * @author   Jongyoon Lee
 * @version  1.10, 11/10/96
 */

public class LogAnalyzerServlet extends HttpServlet {

    static Hashtable months = new Hashtable();

    ServletStub stub;
    String logfile;
    RandomAccessFile access_log;
    Vector entries;
    CountList counts;
    int entryPosition;
    long newlength;

    Hashtable knownEntries;

    static {
        months.put("Jan", "1");
        months.put("Feb", "2");
        months.put("Mar", "3");
        months.put("Apr", "4");
        months.put("May", "5");
        months.put("Jun", "6");
        months.put("Jul", "7");
        months.put("Aug", "8");
        months.put("Sep", "9");
        months.put("Oct", "10");
        months.put("Nov", "11");
        months.put("Dec", "12");
        months.put("1", "31");
        months.put("2", "28");
        months.put("3", "31");
        months.put("4", "30");
        months.put("5", "31");
        months.put("6", "30");
        months.put("7", "31");
        months.put("8", "31");
        months.put("9", "30");
        months.put("10", "31");
        months.put("11", "30");
        months.put("12", "31");
    }

    /**
     * usual constructor
     */
    public LogAnalyzerServlet() {
    }


    /**
     * initializes the servlet
     */
    public void init(ServletStub stub) throws ServletException {
	super.init(stub);
	entries = new Vector();
	counts = new CountList();
	entryPosition = 0;
	knownEntries = new Hashtable();
	logfile = getInitParameter("logfile");
    }


    private void mapEntry(Hashtable ht) {
	ht.put("edu", new Integer(0));
	ht.put("com", new Integer(0));
	ht.put("gov", new Integer(0));
	ht.put("net", new Integer(0));
	ht.put("00", new Integer(0));
	ht.put("01", new Integer(0));
	ht.put("02", new Integer(0));
	ht.put("03", new Integer(0));
	ht.put("04", new Integer(0));
	ht.put("05", new Integer(0));
	ht.put("06", new Integer(0));
	ht.put("07", new Integer(0));
	ht.put("08", new Integer(0));
	ht.put("09", new Integer(0));
	ht.put("10", new Integer(0));
	ht.put("11", new Integer(0));
	ht.put("12", new Integer(0));
	ht.put("13", new Integer(0));
	ht.put("14", new Integer(0));
	ht.put("15", new Integer(0));
	ht.put("16", new Integer(0));
	ht.put("17", new Integer(0));
	ht.put("18", new Integer(0));
	ht.put("19", new Integer(0));
	ht.put("20", new Integer(0));
	ht.put("21", new Integer(0));
	ht.put("22", new Integer(0));
	ht.put("23", new Integer(0));
    }


    /**
     * reads in the access log and update the internal
     * data structure.
     */
    public Vector updateEntries(long offset) throws IOException {

	int pos;
	String date, month, year, olddate;
	Date dt;
	String datehash;

	access_log = new RandomAccessFile(logfile, "r");
	if (access_log.length() == 0) {
	    throw new IOException();
	}

	newlength = access_log.length();

	try {
	    access_log.seek(offset);
	    String inputLine = access_log.readLine();
	    olddate = "";
	    Hashtable ht;
	    while (inputLine != null) {
		//entries.addElement(inputLine);
		try {
		    pos = inputLine.indexOf('[');
		    date = inputLine.substring(pos+1, pos+3);
		    month = (String) months.get(inputLine.substring(pos+4, pos+7));
		    year = inputLine.substring(pos+10, pos+12);
		    datehash = date+month+year;
		    if (!datehash.equals(olddate)) {
			//System.out.println(inputLine);
			//System.out.println("Setting " + datehash);
			ht = new Hashtable();
			knownEntries.put(datehash, ht);
			mapEntry(ht);
		    } else {
			ht = (Hashtable) 
			    knownEntries.get(String.valueOf(datehash));
		    }
		    olddate = datehash;
		    matchHosts(inputLine, ht);
		    matchHours(inputLine, ht);
		    //countURL(inputLine);
		} catch (Exception e) {
		    //e.printStackTrace();
		}
		inputLine = access_log.readLine();
	    }
	} catch (IOException e) {
	    System.out.println("Error reading log file");
	}

	access_log.close();

	Vector retval = new Vector();
	int newdata = entries.size() - entryPosition;

	retval.addElement(Long.toString(newlength));

	entryPosition = entries.size();

	return retval;
    }


    private void countURL(String input) {
	int startpos = input.indexOf('"') + 5;
	//String sub = input.substring(startpos);
	int lastp1 = input.indexOf("HTTP");
	int lastp2 = input.indexOf('?', startpos);
	int lastpos = (lastp2 != -1) ? lastp2 : lastp1;
	//lastpos += startpos;
	try {
	    String url = input.substring(startpos, lastpos);
	    if (counts.find(url)) {
		counts.update(url);
	    } else {
		counts.add(url);
	    }
	    //System.out.println("Counting " + url);
	} catch (Exception e) {
	}
    }


    private void matchHosts(String input, Hashtable ht) {
	int lastpos = input.indexOf(' ');
	String subst = input.substring(lastpos - 3, lastpos);

	if (ht.containsKey(subst)) {
	    //System.out.println(input + " contains " + subst);
	    Integer i = (Integer) ht.get(subst);
	    ht.put(subst, new Integer((i.intValue())+1));
	}
    }


    private void matchHours(String input, Hashtable ht) {
	int begin = input.indexOf('[');
	String subst = input.substring(begin+13, begin+15);

	if (ht.containsKey(subst)) {
	    //System.out.println(input + " contains " + subst);
	    Integer i = (Integer) ht.get(subst);
	    ht.put(subst, new Integer((i.intValue())+1));
	}
    }


    /**
     * given a target string, match pattern in certain field
     */
    public boolean matchPattern(String target, String pattern, String field) {
	int targetLength = target.length();
	int patternLength = pattern.length();

	if (patternLength > targetLength) {
	    return false;
	}

	for (int i = 0; i <= targetLength - patternLength; i++) {
	    if (target.regionMatches(true, i, pattern, 0, patternLength)) {
		return true;
	    }
	}
	return false;
    }


    /**
     * fetches the counts of given pattern
     */
    public Vector countEntries(String pattern, String field, String type) {
	Date today = new Date();
	int year = today.getYear();
	int month = today.getMonth() + 1;
	int date = today.getDate();
	int counter;
	String hashcode;
	int sum = 0;

	if (type.equals("day")) {
	    counter = 1;
	} else if (type.equals("week")) {
	    counter = 7;
	} else if (type.equals("month")) {
	    counter = 30;
	} else {
	    counter = 365;
	}

	while (counter > 0) {
	    try {
		hashcode = String.valueOf(date) +
		    String.valueOf(month) + String.valueOf(year);
		if (date < 10) {
		    hashcode = "0" + hashcode;
		}
		//System.out.println("Getting " + hashcode);
		Hashtable ht = (Hashtable) knownEntries.get(hashcode);
		sum += countDay(pattern, field, ht);
		//System.out.println("sum is " + sum);
		if (--date < 1) {
		    date = Integer.parseInt((String)
					    months.get(String.valueOf(--month)));
		    if (month < 1) {
			month = 12;
			year--;
		    }
		}
	    } catch (Exception e) {
		//e.printStackTrace();
	    }
	    counter--;
	}

	Vector vt = new Vector();
	vt.addElement(Integer.toString(sum));

        return vt;
    }

    private int countDay(String pattern, String field, Hashtable ht) {

	String target;
	int count = 0;

	if (ht.containsKey(pattern)) {
	    Integer result = (Integer) ht.get(pattern);
	    return result.intValue();
	}

	for (int i = 0; i < entryPosition; i++) {
	    target = (String) entries.elementAt(i);
	    if (matchPattern(target, pattern, field)) {
		//System.out.println(pattern + " : " + target);
		count++;
	    }
	}

	return count;
    }


    /**
     * unimplemented
     */
    public Vector getHotPages() {
	int counter = 0;
	CList node = counts.head;
	Vector retval = new Vector();

	while (counter < 10 && node != null) {
	    retval.addElement(node.key);
	    retval.addElement(new Integer(node.value));
	    counter++;
	    node = node.next;
	}

	return retval;
    }

    /**
     * returns the result out to ServletOutputStream
     */
    public void returnResult(ServletOutputStream out, Vector retval) {
	Enumeration elem = retval.elements();
	DBDataEncoder encoder = new DBDataSimpleEncoder(out);

	while (elem.hasMoreElements()) {
	    encoder.write(elem.nextElement().toString());
	    //out.println(elem.nextElement().toString());
	}
    }

    /**
     * overriding the service method for servlet
     */
    public void service(HttpServletRequest req, HttpServletResponse res)
	 throws ServletException, IOException
    {

	try {
	res.setStatus(res.SC_OK);
	res.setContentType("text/plain");

	String action = req.getParameter("action");
	Vector retval;

	if (action.equals("update")) {
	    long offset = Long.parseLong(req.getParameter("offset"));
	    retval = updateEntries(offset);
	} else if (action.equals("count")) {
	    String pattern = req.getParameter("pattern");
	    String field = req.getParameter("field");
	    String type = req.getParameter("type");
	    retval = countEntries(pattern, field, type);
	    /*
	} else if (action.equals("hotpage")) {
	    retval = getHotPages();
	    */
	} else {
	    retval = new Vector();
	}

	returnResult((ServletOutputStream) res.getOutputStream(), retval);

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public String getServletInfo() {
	return "Log Analyzer Servlet";
    }
}

class CountList {

    CList head;
    CList tail;

    public CountList() {
	head = null;
	tail = null;
    }

    public void add(Object obj) {
	CList link = new CList(obj);
	if (head == null) {
	    head = link;
	    tail = link;
	} else {
	    tail.next = link;
	    link.prev = tail;
	    tail = link;
	}
    }

    public boolean find(Object obj) {
	CList link = head;
	while (link != null) {
	    if (link.key.equals(obj)) {
		return true;
	    }
	    link = link.next;
	}
	return false;
    }

    public void update(Object obj) {
	//System.out.println("Updating " + obj.toString());
	CList node = head;
	while (node != null) {
	    if (node.key.equals(obj)) {
		node.value++;
		System.out.println("Found " + node.key + node.value);
		if (node == head) {
		    return;
		}
		CList oldprev = node.prev;
		CList oldnext = node.next;
		CList newprev = null, newnext = null;
		CList prev = node.prev;
		while (prev != head && node.value > prev.value) {
		    prev = prev.prev;
		}
		System.out.println("Found prev " + prev.key + prev.value);
		if (prev == head) {
		    if (node.value > prev.value) {
			// node is new head
			head.prev = node;
			node.next = head;
		    }
		} else {
		    newprev = prev;
		    newnext = prev.next;
		}
		if (oldprev != null) {
		    oldprev.next = oldnext;
		}
		if (oldnext != null) {
		    oldnext.prev = oldprev;
		}
		if (newprev != null) {
		    newprev.next = newnext;
		}
		if (newnext != null) {
		    newnext.prev = newprev;
		}
		break;
	    } // if node.key.equals(obj)
	    node = node.next;
	} // while (node != null)
    }
}


class CList {
    public Object key;
    public int value;
    public CList prev;
    public CList next;

    public CList(Object key) {
	this.key = key;
	value = 1;
	prev = null;
	next = null;
    }
}
