/*
 * @(#)MovieFinderServlet.java	1.18 96/11/10 Scott Atwood, Jongyoon Lee
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.  All Rights reserved
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file copyright.html
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.io.*;
import java.net.*;
import java.sql.*;
import java.servlet.*;
import java.servlet.http.*;


/**
 * MovieFinderServlet
 * Back-end of an Applet-Servlet pair that demonstrates
 * the powerful combination of Jeeves servlet technology
 * with JDBC database interface.
 * 
 * @author   Scott Atwood
 * @author   Jongyoon Lee
 * @version  1.18, 11/10/96
 */

public class MovieFinderServlet extends HttpServlet {

    // TODO:
    // all these class variables shouldn't be shared eventually
    // since they are connection dependent.
    String driverURL;
    String username;
    String passwd;
    String owner;
    String catalog = null;
    String schema = null;

    Connection con;
    DatabaseMetaData metaData;

    /**
     * Creates a new MovieFinderServlet servlet
     */
    public MovieFinderServlet() {
    }

    /**
     * Initializes the database
     */
    public void init(ServletStub stub) throws ServletException {

        SQLWarning warn;
	Object timer = new Object();
        
        super.init(stub);

        driverURL = getInitParameter("driverURL");
        username = getInitParameter("username");
        passwd = getInitParameter("password");
        owner = getInitParameter("owner");

        try {    
	    Class.forName("jdbc.odbc.JdbcOdbcDriver");
	    //Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");

	    // uncomment for debugging
	    //DriverManager.setLogStream(java.lang.System.out);

            con = DriverManager.getConnection(driverURL, username, passwd);

	    if ( (warn = con.getWarnings()) != null ) {
		while(warn != null) {
		    log("SQLWarning: " + warn.getSQLState() + '\t' +
			warn.getMessage() + '\t' +
			warn.getErrorCode() + '\t');
		    warn = warn.getNextWarning();
		}
	    }
	    metaData = con.getMetaData();
	    System.out.println ("\nConnected to " + metaData.getURL());
	    System.out.println ("Driver       " + metaData.getDriverName());
	    System.out.println ("Version      " + metaData.getDriverVersion());
        } catch (ClassNotFoundException e) {
	    throw new ServletException("init");
	} catch (SQLException e) {
	    try {
	        con = DriverManager.getConnection(driverURL, username, passwd);
		metaData = con.getMetaData();
		System.out.println ("\nConnected to " + metaData.getURL());
		System.out.println ("Driver       " + metaData.getDriverName());
		System.out.println ("Version      " + metaData.getDriverVersion());
	    } catch (SQLException ee) {
	        ee.printStackTrace();
	        while(e != null) {
		    log("SQLException: " + e.getSQLState() + '\t' +
			e.getMessage()  + '\t' +
			e.getErrorCode());
		    e = e.getNextException();
		}
		throw new ServletException("init");
	    }
	}

    }

    /**
     * Services a single client request
     *
     * @param req the servlet request
     * @param res the servlet response
     * @exception IOExceptoin If an I/O error has occurred
     */
    public void service(HttpServletRequest req, HttpServletResponse res)
	 throws ServletException, IOException
    {
        String action = req.getParameter("action");

	try {
	    if(action.equals("query")) {
		query(req, res);
	    } else if(action.equals("metadata")){
		metadata(req, res);
	    }
	} catch (SQLException e) {
            while(e != null) {
		log("SQLException: " + e.getSQLState() + '\t' +
		    e.getMessage() + '\t' +
		    e.getErrorCode() + '\t');
		e = e.getNextException();
            }
	} catch (MovieFinderServletException e) {
	}
    }

    /**
     * Cleans up database connection
     */
    public void destroy() {
        try {
            con.close();
        } catch (SQLException e) {
            while(e != null) {
		log("SQLException: " + e.getSQLState() + '\t' +
		    e.getMessage() + '\t' +
		    e.getErrorCode() + '\t');
		e = e.getNextException();
            }
        } catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * executes the given SQL query and returns the result
     */
    public void query(HttpServletRequest req, HttpServletResponse res) 
	 throws IOException, MovieFinderServletException, SQLException
    {
        Statement stmt = con.createStatement();
        String query;
        ResultSet rs;
        ServletOutputStream out;
        int start;
        int end;

        res.setStatus(res.SC_OK);
        res.setContentType("text/plain");
        out = res.getOutputStream();

        start = (req.getParameter("start") != null) ? 
	    Integer.parseInt(req.getParameter("start")) :
	    0;
		
        end = (req.getParameter("end") != null) ? 
	    Integer.parseInt(req.getParameter("end")) :
	    Integer.MAX_VALUE;
		
	query = req.getParameter("sqlquery");
	if (query == null) {
	    String select = req.getParameter("select");
	    String from = req.getParameter("from");
	    String where = req.getParameter("where");
	    if(select == null || from == null) {
		throw new MovieFinderServletException();
	    }
	    query = "SELECT " + select + " FROM " + from +
		( (where != null) ? " WHERE " + where : "");
	}

        rs = stmt.executeQuery(query);

	printToStream(rs, out, start, end);

	rs.close();
	stmt.close();
    }


    /**
     * print the data to the http stream
     */
    public void printToStream(ResultSet rs, ServletOutputStream out, 
            int start, int end) 
    throws IOException, MovieFinderServletException, SQLException {

	ResultSetMetaData rsmd;
	int numRows;
	int dataType;
	int numCols;
	String columnName;
	DBDataEncoder encoder;

	rsmd = rs.getMetaData();
	numCols = rsmd.getColumnCount();

	numRows = (end - start >= 0) ? end - start : 0;

	encoder = new DBDataSimpleEncoder(out);

	for(int i=0; i < start && rs.next(); i++) {}
		
	for(int row = 0; row < numRows && rs.next(); row++) {
	    for(int col = 1; col <= numCols; col++) {
		dataType = rsmd.getColumnType(col);
		columnName = rsmd.getColumnName(col);
				
		switch(dataType) {
		case Types.DATE:
		    java.sql.Date date = rs.getDate(col);
		    encoder.write(date);
		    break;
		case Types.TIME:
		    java.sql.Time time = rs.getTime(col);
		    encoder.write(time);
		    break;
		case Types.TIMESTAMP:
		    java.sql.Timestamp timestamp = rs.getTimestamp(col);
		    encoder.write(timestamp);
		    break;
		case Types.CHAR:
		case Types.VARCHAR:
		case Types.LONGVARCHAR:
		    String str = rs.getString(col);
		    encoder.write(str);
		    break;
		case Types.NUMERIC:
		case Types.DECIMAL:
		    java.sql.Numeric numeric = rs.getNumeric(col, 10);
		    encoder.write(numeric);
		    break;
		case Types.BIT:
		    boolean bit = rs.getBoolean(col);
		    encoder.write(new Boolean(bit));
		    break;
		case Types.TINYINT:
		    byte tinyint = rs.getByte(col);
		    encoder.write(new Integer(tinyint));
		    break;
		case Types.SMALLINT:
		    short smallint = rs.getShort(col);
		    encoder.write(new Integer(smallint));
		    break;
		case Types.INTEGER:
		    int integer = rs.getInt(col);
		    encoder.write(new Integer(integer));
		    break;
		case Types.BIGINT:
		    long bigint = rs.getLong(col);
		    encoder.write(new Long(bigint));
		    break;
		case Types.REAL:
		    float real = rs.getFloat(col);
		    encoder.write(new Float(real));
		    break;
		case Types.FLOAT:
		case Types.DOUBLE:
		    double longreal = rs.getDouble(col);
		    encoder.write(new Double(longreal));
		    break;
		case Types.BINARY:
		case Types.VARBINARY:
		case Types.LONGVARBINARY:
		    byte[] binary = rs.getBytes(col);
		    encoder.write(new String(binary, 0));
		    break;
		}
	    }
	}
    }
	

    private void printTable(ResultSet rs, ServletOutputStream out)
    throws IOException, MovieFinderServletException, SQLException {
	DBDataEncoder encoder = new DBDataSimpleEncoder(out);
	while (rs.next()) {
	    if (rs.getString(2).equalsIgnoreCase(owner)) {
		encoder.write(rs.getString(3));
		//out.print(rs.getString(3) + "\n");
	    }
	}

    }


    /**
     * executes the given SQL query and returns the metadata
     */
    public void metadata(HttpServletRequest req, HttpServletResponse res) 
	 throws IOException, MovieFinderServletException, SQLException
    {
        Statement stmt = con.createStatement();
        String query;
        ResultSet rs;
	ResultSetMetaData rsmd;
        ServletOutputStream out;
	int dataType;
	int numCols;
	String columnName;

        res.setStatus(res.SC_OK);
        res.setContentType("text/plain");
        out = (ServletOutputStream) res.getOutputStream();

	query = req.getParameter("meta");
	if (query == null) {
	    // shouldn't happen.
	}

	if (query.equals("table")) {
	    String types[] = new String[5];
	    types[0] = "TABLE";
	    rs = metaData.getTables(catalog, schema, "%", types);
	} else {
	    String types[] = new String[5];
	    types[0] = "TABLE";
	    rs = metaData.getTables(catalog, schema, "%", types);
	}

	printTable(rs, out);

	/*
	rsmd = rs.getMetaData();
	numCols = rsmd.getColumnCount();

	for(int col = 1; col <= numCols; col++) {
	    dataType = rsmd.getColumnType(col); 
	    columnName = rsmd.getColumnName(col);

	    switch(dataType){
	    case Types.DATE:
		out.print("DATE");
		break;
	    case Types.TIME:
		out.print("TIME");
		break;
	    case Types.TIMESTAMP:
		out.print("TIMESTAMP");
		break;
	    case Types.CHAR:
		out.print("CHAR");
		break;
	    case Types.VARCHAR:
		out.print("VARCHAR");
		break;
	    case Types.LONGVARCHAR:
		out.print("LONGVARCHAR");
		break;
	    case Types.NUMERIC:
		out.print("NUMERIC");
		break;
	    case Types.DECIMAL:
		out.print("DECIMAL");
		break;
	    case Types.BIT:
		out.print("BIT");
		break;
	    case Types.TINYINT:
		out.print("TINYINT");
		break;
	    case Types.SMALLINT:
		out.print("SMALLINT");
		break;
	    case Types.INTEGER:
		out.print("INTEGER");
		break;
	    case Types.BIGINT:
		out.print("BIGINT");
		break;
	    case Types.REAL:
		out.print("REAL");
		break;
	    case Types.FLOAT:
		out.print("FLOAT");
		break;
	    case Types.DOUBLE:
		out.print("DOUBLE");
		break;
	    case Types.BINARY:
		out.print("BINARY");
		break;
	    case Types.VARBINARY:
		out.print("VARBINARY");
		break;
	    case Types.LONGVARBINARY:
		out.print("LONGVARBINARY");
		break;
	    }
	    out.print("\t");
	    out.print(columnName);
	    out.print("\n");
	}
	out.print("\n");

	*/

	rs.close();
	stmt.close();
    }
}

class MovieFinderServletException extends Exception {
}
