/*
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "COPYRIGHT"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


// Simple JavaIDL Jeeves servlet.  Listens to incoming requests on port
// 4490 and echos the input.

import echo.*;
import java.servlet.*;
import java.io.*;

public
class EchoServlet extends GenericServlet {

    public void init() {

	// We use port 4490 to accept incoming Java-java connections.
	sunw.door.Orb.initialize(4490);

	// Create and publish an echo IDL objRef.
	try {
	    // The EchoSkeleton was created by the IDL compiler
	    EchoRef echo = EchoSkeleton.createRef(new EchoImpl());
	    sunw.door.Orb.publish("echo", echo);
	} catch (Exception ex) {
	    System.err.println("Couldn't bind object in naming context: " + ex);
	    ex.printStackTrace();
	    return;
	}
	System.err.println("setup and bound Echo server object OK");

    }

    // When an http request comes in for this servlet, return the
    // JavaIDL name for the echo object
    public void service(ServletRequest req, ServletResponse res) 
    {
	res.setContentType("text/plain");
	String hostname;
	PrintStream out = new PrintStream(res.getOutputStream());
        try {
	    hostname = java.net.InetAddress.getLocalHost().getHostName();
	} catch (Exception ex) {
	    out.println("Couldn't get hostname: " + ex);
	    ex.printStackTrace();
	    return;
	}
	out.println("idl:sunw.door://" + hostname + ":4490/echo");
    }

    public String getServletInfo() 
    {
      return "This example servlet acts as a JavaIDL server";
    }
}

