/*
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 0.8
 */

package rmiexample;

import java.io.*;
import java.awt.*;
import java.util.Date;

import java.rmi.registry.*;
import java.rmi.*;

import java.net.*;

import rmiexample.Counter;

public class CounterApplet extends java.applet.Applet implements Runnable {
    TextField hostfield;
    TextField argfield;
    TextField resultfield;
    Button lookupbutton;
    Button incrbutton;
    Button runbutton;
    Counter counter = null;
    int steps = 0;
    int totalsteps = 0;
    int initialsteps = 0;
    Thread runthread = null;
    Thread showthread = null;
    int startsteps, endsteps;
    Date startdate, enddate;

    public synchronized void init() {
	String host = null;

	host = getParameter("url");
	if (host == null) {
	    try {
		Object context = System.getSecurityManager().getSecurityContext();
		if (context != null && context instanceof URL) {
		    host = "//" + ((URL)context).getHost() + "/servletcounter";
		} else {
		    host = "//" + InetAddress.getLocalHost().getHostName();
		    host += "/servletcounter";
		}
	    } catch (java.net.UnknownHostException ex) {
		showStatus("Default host is unknown.");
	    }
	}

	// Create the host label and textfield as  panel using the default flowlayout

 	setLayout(new GridLayout(0,3));
	add(new Label("URL:", Label.RIGHT));
	hostfield = new TextField(30);
	hostfield.setText(host);
	add(hostfield);
	add(lookupbutton = new Button("Lookup"));


	// Create the values group
	add(new Label("Times incremented:", Label.RIGHT));
	argfield = new TextField(30);
	argfield.setEditable(false);
	add(argfield);
	add(incrbutton = new Button("Increment"));

	add(new Label("Total increments:", Label.RIGHT));
	resultfield = new TextField(50);
	resultfield.setEditable(false);
	add(resultfield);
	add(runbutton = new Button("Run"));

	getCounterServer(hostfield.getText());
	if (counter != null) {
	    doCounter();
	}
    }

    public boolean action(Event ev, Object obj) {
	if (ev.target == hostfield) {
	    getCounterServer((String)ev.arg);
	} else if (ev.target == lookupbutton) {
	    getCounterServer(hostfield.getText());
	} else if (ev.target == incrbutton) {
	    doCounter();
	} else if (ev.target == runbutton) {
	    doRun();
	} 

	return true;
    }

    public boolean handleEvent(Event ev) {
	boolean rc = super.handleEvent(ev);

	return rc;
    }

    public void getCounterServer(String where) {
	URL url = null;
	try {
	    Remote obj = Naming.lookup(where);
	    if (obj instanceof Counter) {
		counter = (Counter)obj;
		steps = initialsteps = totalsteps = 0;
		argfield.setText(Integer.toString(steps));
		resultfield.setText(Integer.toString(totalsteps-initialsteps));
		showStatus("");
	    } else {
		showStatus("Named object is not an Counter server.");
	    }
	} catch (NotBoundException ex) {
	    showStatus("Name not bound.");
	} catch (java.rmi.UnknownHostException ex) {
	    showStatus("Host unknown in url " + where);
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus(ex.getMessage());
	} catch (java.net.MalformedURLException ex) {
	    showStatus(ex.getMessage() + where);
	}
    }

    public void doCounter() {
	try {
	    if (counter != null) {
		argfield.setText(Integer.toString(++steps));
		totalsteps = counter.inc();
		if (initialsteps == 0)
		    initialsteps = totalsteps-1;

		resultfield.setText(Integer.toString(totalsteps-initialsteps));
		showStatus("");
	    }
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus("RemoteException during counter.");
	}
    }

    /** Start the async counter if not running, else stop it
     */
    public void doRun() {
	if (runthread == null || showthread == null) {
	    runbutton.setLabel("Stop");
	    runthread = new Thread(this);
	    startdate = new Date();
	    startsteps = steps;
	    showthread = new Thread(this);
	    runthread.start();
	    showthread.start();

	} else {
	    runthread.stop();
	    showthread.stop();
	    runbutton.setLabel("Run");
	    runthread = null;
	    enddate = new Date();
	    endsteps = steps;
	    long duration = enddate.getTime() - startdate.getTime();
	    showStatus("Elapsed time " + (float)duration/1000.0 + " sec " + 
		       ((endsteps - startsteps) * 1000) / (int)duration +
		       " steps/sec, " +
		       ((totalsteps - initialsteps) * 1000) / (int) duration +
		       " total steps/sec.");
	    argfield.setText(Integer.toString(steps));
	    resultfield.setText(Integer.toString(totalsteps-initialsteps));
	}
    }
    public void run() {
	
	if (Thread.currentThread() == runthread) {
	    runsteps();
	} else {
	    showsteps();
	}
    }

    /** While running, increment the counter as fast as possible.
     */
    private void runsteps() {

	try {
	    showStatus("");
	    if (counter != null) {
		while (true) {
		    ++steps;
		    totalsteps = counter.inc();
		    if (initialsteps == 0)
			initialsteps = totalsteps-1;
		}
	    }
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus("RemoteException during increment.");
	}
    }

    /** Update the display every second, with the current stats
     */
    private void showsteps() {
	while (true) {
	    Date firstdate = new Date();
	    int firststeps = steps;

	    try {
		Thread.sleep(1000);	// sleep for a sec
	    } catch (InterruptedException ex) {
	    }

	    Date lastdate = new Date();
	    int laststeps = steps;
	    long duration = lastdate.getTime() - firstdate.getTime();
	    showStatus(((laststeps - firststeps) * 1000) / (int)duration +
		       " steps/second");
	    argfield.setText(Integer.toString(steps));
	    resultfield.setText(Integer.toString(totalsteps-initialsteps));
	}
    }
}
