/*
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 0.8
 */

package rmiexample;

import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;


public class CounterImpl
	extends UnicastRemoteObject
	implements Counter
{
    private int num = 0;
    
    public CounterImpl() throws RemoteException
    {}
     
    public void noop() throws RemoteException
    {
    }

    public synchronized int reset() {
	int v = num;
	num = 0;
	return v;
    }

    public synchronized int inc() {
	num++;
	System.out.println("num:"+num);
	return num;
    }

    public static void main(String args[]) 
    {
	// Create and install the security manager
	System.setSecurityManager(new RMISecurityManager());

	try {

	    Registry registry = LocateRegistry.getRegistry(getHost(args));
	
	    CounterImpl counter = new CounterImpl();
	    registry.rebind(getName(args), counter);
	    
	} catch (Exception e) {
	    System.out.println("CounterImpl.main: an exception occurred: " +
			       e.getMessage());
	    e.printStackTrace();
	}
    }

    private static String getName(String[] args) {
	return (args.length < 1 ? "counter" : args[1]);
    }

    private static String getHost(String[] args) {
	return (args.length < 2 ? null : args[0]);
    }
}
