/*
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 0.8
 */

package rmiexample;

import java.io.IOException;

import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;

import java.servlet.*;

public class CounterRMIServlet
	extends UnicastRemoteObject
	implements Counter, Servlet
{
    private int num = 0;

static {
    if (System.getSecurityManager() == null) {
	System.out.println("security manager is null in main");
	System.setSecurityManager(new RMISecurityManager());
    }
}
    
// Counter & RMI methods
    public CounterRMIServlet() throws RemoteException
    {
	System.out.println("construct");
    }
     
    public void noop() throws RemoteException
    {
    }

    public synchronized int reset() {
	System.out.println("reset");
	int v = num;
	num = 0;
	return v;
    }

    public synchronized int inc() {
	num++;
	System.out.println("num: "+num);
	return num;
    }

// servlet support
    private ServletStub stub;
    public void init(ServletStub stub) throws ServletException {
	System.out.println("init with stub");
	this.stub = stub;
	init();
    }

    public void init() throws ServletException {
	System.out.println("init");
 	log("init");
	try {
	    Registry registry = LocateRegistry.getRegistry(null);
	    registry.rebind("servletcounter", this);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public ServletContext getServletContext() {
	return stub.getServletContext();
    }

    public String getInitParameter(String name) {
	return stub.getInitParameter(name);
    }

    public void log(String msg) {
	getServletContext().log(this, msg);
    }

    public String getServletInfo() {
	return "This is a sample RMI server / Jeeves servlet";
    }

    public void service(ServletRequest req, ServletResponse res) 
    throws ServletException, IOException {
	res.getOutputStream().print("" + inc());
    }

    public void destroy() {
	log("destroy");
    }
}
