# @(#)Makefile.sol	1.6 97/02/05
#
# Solaris-specific makefile for building optional native library support.
# Use this as a starting point for porting to other UNIX platforms.
#

# Path to top of Java Web Server distribution tree
TOP =		..

# Name of the library we're making (given to System.loadLibrary)
LIBNAME = 	server

# Location of Java Developer's Kit (and runtime)
JAVA_HOME =	/opt/java

# Java runtime flags
JAVAH =		$(JAVA_HOME)/bin/javah
CLASSPATH =	$(TOP)/classes:$(TOP)/lib/classes.zip:$(JAVA_HOME)/classes:$(JAVA_HOME)/lib/classes.zip

# OS and machine type
OS =		solaris
ARCH:sh =	uname -p
OBJ = 		$(OS)/$(ARCH)

# Preprocessor, compiler, linker, and related flags
CC =		cc
CFLAGS =	-fast -Kpic
CPPFLAGS =	-I$(OS) -I$(JAVA_HOME)/include -I$(JAVA_HOME)/include/$(OS)
LDFLAGS =	-G -h $(LIBNAME)
INSTALL =	/usr/sbin/install

LIBDIR =	$(TOP)/lib/$(OBJ)

LIBRARY =	$(OBJ)/lib$(LIBNAME).so
OBJS =		$(OBJ)/server.o \
		$(OBJ)/UNIXUser.o \
		$(OBJ)/UNIXUserEnumeration.o \
		$(OBJ)/sun_server_ServerProcess.o \
		$(OBJ)/sun_server_realm_UNIXUser.o \
		$(OBJ)/sun_server_realm_UNIXUserEnumeration.o

default all: $(LIBRARY)

$(LIBRARY): $(OBJS)
	$(LINK.c) -o $@ $(OBJS)

$(OBJ)/server.o: server.c $(OS)/sun_server_ServerProcess.h
	$(COMPILE.c) -o $@ server.c

$(OBJ)/UNIXUser.o: UNIXUser.c $(OS)/sun_server_realm_UNIXUser.h
	$(COMPILE.c) -o $@ UNIXUser.c

$(OBJ)/UNIXUserEnumeration.o: UNIXUserEnumeration.c \
		$(OS)/sun_server_realm_UNIXUserEnumeration.h
	$(COMPILE.c) -o $@ UNIXUserEnumeration.c


$(OBJ)/sun_server_ServerProcess.o: $(OS)/sun_server_ServerProcess.c
	$(COMPILE.c) -o $@ $(OS)/sun_server_ServerProcess.c
$(OS)/sun_server_ServerProcess.c:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) -stubs sun.server.ServerProcess
$(OS)/sun_server_ServerProcess.h:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) sun.server.ServerProcess

$(OBJ)/sun_server_realm_UNIXUser.o: $(OS)/sun_server_realm_UNIXUser.c
	$(COMPILE.c) -o $@ $(OS)/sun_server_realm_UNIXUser.c
$(OS)/sun_server_realm_UNIXUser.c:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) \
	    -stubs sun.server.realm.UNIXUser
$(OS)/sun_server_realm_UNIXUser.h:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) sun.server.realm.UNIXUser

$(OBJ)/sun_server_realm_UNIXUserEnumeration.o: \
		$(OS)/sun_server_realm_UNIXUserEnumeration.c
	$(COMPILE.c) -o $@ $(OS)/sun_server_realm_UNIXUserEnumeration.c
$(OS)/sun_server_realm_UNIXUserEnumeration.c:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) \
	    -stubs sun.server.realm.UNIXUserEnumeration
$(OS)/sun_server_realm_UNIXUserEnumeration.h:
	$(JAVAH) -d $(OS) -classpath $(CLASSPATH) \
		sun.server.realm.UNIXUserEnumeration

install: $(LIBRARY)
	$(INSTALL) -d $(LIBDIR)
	$(INSTALL) -f $(LIBDIR) $(LIBRARY)

clean:
	rm -rf $(OS)
	
.INIT:
	-@mkdir -p $(OBJ)

.KEEP_STATE:

