/*
 * @(#)server.c	1.7 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */


/*
 * Optional native code support for "service" base class.
 *
 * XXX This is now misnamed - the functionality is part of the process,
 * not the service running within it.  This artifact of API changes
 * will be fixed.
 */

#include <pwd.h>
#include <grp.h>
#include <errno.h>
#include <stdio.h>

#include "sun_server_ServerProcess.h"

long
sun_server_ServerProcess_setUser(Hsun_server_ServerProcess *Hthis, HString *Huser)
{
    if (Huser == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else {
	char buf[128];
	char *s = makeCString(Huser);
	if (isdigit(s[0])) {
	    if (setuid(atoi(s)) == -1) {
		if (errno == EINVAL) {
		    jio_snprintf(buf, sizeof(buf), "Invalid user id: %s", s);
		    SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		}
		return FALSE;
	    }
	} else {
	    struct passwd *ent = getpwnam(s);
	    if (ent == 0) {
		jio_snprintf(buf, sizeof(buf), "Invalid user: %s", s);
		SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		return FALSE;
	    } else if (setuid(ent->pw_uid) == -1) {
		return FALSE;
	    }
	}
    }
    return TRUE;
}

long
sun_server_ServerProcess_setGroup(Hsun_server_ServerProcess *Hthis, HString *Hgroup)
{
    if (Hgroup == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else {
	char buf[128];
	char *s = makeCString(Hgroup);
	if (isdigit(s[0])) {
	    if (setgid(atoi(s)) == -1) {
		if (errno == EINVAL) {
		    jio_snprintf(buf, sizeof(buf), "Invalid group id: %s", s);
		    SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		}
		return FALSE;
	    }
	} else {
	    struct group *ent = getgrnam(s);
	    if (ent == 0) {
		jio_snprintf(buf, sizeof(buf), "Invalid group: %s", s);
		SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		return FALSE;
	    } else if (setgid(ent->gr_gid) == -1) {
		return FALSE;
	    }
	}
    }
    return TRUE;
}

long
sun_server_ServerProcess_isRoot(Hsun_server_ServerProcess *Hthis)
{
    if (getuid() == 0 || geteuid() == 0) {
	return TRUE;
    } else {
	return FALSE;
    }
}
