/*
 * @(#)ChatServlet.java	1.16 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */


import java.io.*;
import java.util.*;
import java.servlet.*;


/**
 * This is an example of a chat Servlet
 */
public class ChatServlet implements Servlet {
ServletConfig conf;

    /**
     * Vector of client information
     */  
    Vector clientVector = new Vector();

    public void init(ServletConfig conf) throws ServletException {
	this.conf = conf;
        //System.out.println("ChatServlet.init: enter");
    }

    public ServletConfig getServletConfig() {
	return conf;
    }

    public void service(ServletRequest req, ServletResponse res) 
	throws ServletException, IOException {
        //System.out.println("ChatServlet.service: enter");
        converse(req.getInputStream(), res.getOutputStream());
    }

    /**
     * when you get a new connection
     */
    void converse(InputStream clientInput, OutputStream clientOutput) {
        ClientInfo clientInfo;
	String line; 
        DataInputStream dis;
        boolean firstLine = true;

        clientInfo = new ClientInfo();
        clientInfo.clientOutput = new PrintStream(clientOutput);
        dis = new DataInputStream(clientInput);
	try {
	    while (true) {
	        line = dis.readLine();
                //System.out.println("line:");
                //System.out.println(line);
                if (line == null) {
                    break;
                }
                if (firstLine == true) { 
                    // the first line is the name
                    clientInfo.name = line;
                    firstLine = false;
	            addListener(clientInfo);
                } else {
                    // the subsequent lines are input to the chat room
		    addString(clientInfo.name + ": " + line);
		}
	    }
	} catch(Exception e) {
            log("ChatServerHandler.converse: " + e);
	}
	removeListener(clientInfo);
    }

    /**
     * add a listener to the chat room
     */
    synchronized void addListener(ClientInfo clientInfo) throws Exception {
        //System.out.println("ChatServlet.addListener: enter");
        log("ChatServlet.addListener: enter");
        log("ChatServlet.addListener: " + clientInfo.name + " entered");
        // let everyone else know
	addString(clientInfo.name + " has just entered the room.\n");
        clientVector.addElement(clientInfo);
        // let the current listener know who is in th room
	clientInfo.clientOutput.print("Welcome to Chat demo\n");
	clientInfo.clientOutput.flush();
    }

    /**
     * remove a listener from the chat room
     */
    synchronized void removeListener(ClientInfo clientInfo) {
        log("ChatServlet.removeListener: enter");
        log("ChatServlet.removeListener: " + clientInfo.name + " left");
        clientVector.removeElement(clientInfo);
	addString(clientInfo.name + " has left the room.\n");
    }

    /**
     * send a string to all the clients
     */
    synchronized void addString(String s) {
        Enumeration enum;
        ClientInfo clientInfo;

        enum = clientVector.elements();
	while(enum.hasMoreElements()) {
            clientInfo = (ClientInfo) enum.nextElement();
	    try {
	        clientInfo.clientOutput.print(s);
	        clientInfo.clientOutput.print("\n");
	        clientInfo.clientOutput.flush();
	    } catch(Exception e) {
	        removeListener(clientInfo);
	    }
        }
    }

    public String getServletInfo() {
        return "A chat servlet";
    }

    public void destroy() {
	log("destroy");
    }

    public void log(String message) {
	conf.getServletContext().log(getClass().getName() + ": " + message);
    }


    /**
     * Information stored in the server about each client
     */
    class ClientInfo {
	String name;
	PrintStream clientOutput;
    }
}
