/*
 * @(#)EchoClient.java	1.2 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */


// A simple java applet that can run in a Java document and connect
// to an Echo servlet running in a Jeeves web server on a named machine.

import java.awt.*;
import java.io.*;
import java.applet.Applet;
import java.net.URL;
import echo.*;

public class EchoClient extends Applet {

    public void init() {
	// Initialize the applet.
	url = getParameter("url");
	if (url == null) {
	    System.err.println("EchoClient not given a URL");
	    badArgs = true;
	}

    }

    public void start() {
	// (Re)start the applet.
	if (badArgs) {
	    return;
    	}
	error = null;
	showStatus("Connecting to server at " + url);

	try {
	    // First contact the Echo Servlet through http causing it to
	    // start and to return it's JavaIDL URL
	    URL servlet = new URL(url);
	    byte[] javaIDLData = new byte[1000];
	    // Read the result of the HTTP request and create a string from it
	    InputStream javaIDLStream = servlet.openStream();
	    int cnt = javaIDLStream.read(javaIDLData);
	    String javaIDLURL = new String(javaIDLData, 0, 0, cnt-1);
	    try {
	        // Create a JavaIDL object and bind it to the Echo object
	        target = EchoStub.createRef();
	        sunw.corba.Orb.resolve(javaIDLURL, target);
            } catch (Exception ex) {
	        System.err.println("Couldn't resolve " + javaIDLURL + " : " + ex);
	        setError("Couldn't resolve Echo object " + javaIDLURL);
		return;
	    }

	    error = null;
	    repaint();

        } catch (Throwable th) {
            System.err.println("EchoClient.start caught: " + th);
	    th.printStackTrace();
	    setError("EchoClient.start failed : " + th);
	}

	// We do our own layout.
	setLayout(null);
	removeAll();

	echoButton = new Button("Call Echo server");
	add(echoButton);
	echoButton.reshape(15, 10, 110, 25);

	inField = new TextField("Hi there!", 25);
	add(inField);
	inField.reshape(200, 5, 280, 30);

	outField = new TextField("", 25);
	add(outField);
	outField.reshape(200, 40, 280, 30);
		
    }

    public void stop() {
	// Stop the applet.
	target = null;
	removeAll();
    }

    public void paint(Graphics g) {

	if (error != null) {
	    // Something is wrong.  Draw the error message and return.
	    g.drawString(error, 44, 20);
	    return;
	}

	// get our parent to paint the button(s).
	super.paint(g);

	// Draw our count of calls:
        g.drawString(count + " Echo calls", 33, 60);

	// Draw our two message strings:
	g.drawString("Input:", 145, 25);
	g.drawString("Output:", 145, 60);
	

    }

    public boolean action(Event evt, Object arg) {
	if (evt.target != echoButton) {
	    return (false);
	}
	try {
	    if (target != null && inField != null) {
		String input = inField.getText();
		if (input.length() > 100) {
		    input = input.substring(0,100);
		}
	        String output = target.echo(input);
		outField.setText(output);
	        count++;
	    }
	} catch (Exception ex) {
	    System.err.println("Caught exception while invoking Echo: " + ex);
	    setError("exception while talking to target");
	    ex.printStackTrace();
  	}
	repaint();
	return (true);
    }

    void setError(String message) {
	error = message;
	removeAll();
	repaint();
    }


    java.awt.Button echoButton;
    java.awt.TextField inField;
    java.awt.TextField outField;
    int count;
    EchoRef target;
    String error = null;
    boolean badArgs = false;

    // URL for target object
    String url;
}
