/*
 * @(#)CounterApplet.java	1.5 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package rmiexample;

import java.io.*;
import java.awt.*;

import java.rmi.registry.*;
import java.rmi.*;

import java.net.*;

import rmiexample.Counter;

public class CounterApplet extends java.applet.Applet implements Runnable {
    TextField hostfield;
    TextField argfield;
    TextField resultfield;
    Button lookupbutton;
    Button incrbutton;
    Button runbutton;
    Counter counter = null;
    int steps = 0;
    int totalsteps = 0;
    Thread runthread = null;
    Thread showthread = null;
    int startsteps, endsteps;
    long startdate, enddate;

    public synchronized void init() {
	String host = null;

	host = getParameter("url");
	if (host == null) {
	    try {
		URL base = getCodeBase();
		host = "//" + base.getHost() + "/servletcounter";
	    } catch (Exception ex) {
		showStatus("Default host is unknown.");
	    }
	}

	// Create the host label and textfield as  panel using the default flowlayout
 	setLayout(new GridLayout(0,3));
	add(new Label("URL:", Label.RIGHT));
	hostfield = new TextField(30);
	hostfield.setText(host);
	add(hostfield);
	add(lookupbutton = new Button("Lookup"));


	// Create the values group
	add(new Label("Times incremented:", Label.RIGHT));
	argfield = new TextField(30);
	argfield.setEditable(false);
	add(argfield);
	add(incrbutton = new Button("Increment"));

	add(new Label("Local increments:", Label.RIGHT));
	resultfield = new TextField(50);
	resultfield.setEditable(false);
	add(resultfield);
	add(runbutton = new Button("Run"));

	getCounterServer(hostfield.getText());
    }

    public boolean action(Event ev, Object obj) {
	if (ev.target == hostfield) {
	    getCounterServer((String)ev.arg);
	} else if (ev.target == lookupbutton) {
	    getCounterServer(hostfield.getText());
	} else if (ev.target == incrbutton) {
	    doCounter();
	} else if (ev.target == runbutton) {
	    doRun();
	} 

	return true;
    }

    public void getCounterServer(String where) {
	URL url = null;
	try {
	    Remote obj = Naming.lookup(where);
	    if (obj instanceof Counter) {
		counter = (Counter)obj;
		steps = counter.inc();
		totalsteps = 1;
		argfield.setText(Integer.toString(steps));
		resultfield.setText(Integer.toString(totalsteps));
		showStatus("");
	    } else {
		showStatus("Named object is not an Counter server.");
	    }
	} catch (NotBoundException ex) {
	    showStatus("Name not bound.");
	} catch (java.rmi.UnknownHostException ex) {
	    showStatus("Host unknown in url " + where);
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus(ex.getMessage());
	} catch (java.net.MalformedURLException ex) {
	    showStatus(ex.getMessage() + where);
	}
    }

    public void doCounter() {
	try {
	    if (counter != null) {
		steps = counter.inc();
		totalsteps++;
		argfield.setText(Integer.toString(steps));
		resultfield.setText(Integer.toString(totalsteps));

		showStatus("");
	    }
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus("RemoteException during counter.");
	}
    }

    /** Start the async counter if not running, else stop it
     */
    public void doRun() {
	if (runthread == null || showthread == null) {
	    runbutton.setLabel("Stop");
	    runthread = new Thread(this);
	    startdate = System.currentTimeMillis();
	    startsteps = steps;
	    showthread = new Thread(this);
	    runthread.start();
	    showthread.start();

	} else {
	    runthread.stop();
	    showthread.stop();
	    runbutton.setLabel("Run");
	    runthread = null;
	    enddate = System.currentTimeMillis();
	    endsteps = steps;
	    long duration = enddate - startdate;
	    showStatus("Elapsed time " + (float)duration/1000.0 + " sec " + 
		       ((endsteps - startsteps) * 1000) / (int)duration +
		       " steps/sec.");
	    argfield.setText(Integer.toString(steps));
	    resultfield.setText(Integer.toString(totalsteps));
	}
    }
    public void run() {
	
	if (Thread.currentThread() == runthread) {
	    runsteps();
	} else {
	    showsteps();
	}
    }

    /*
     * While running, increment the counter as fast as possible.
     */
    private void runsteps() {

	try {
	    showStatus("");
	    if (counter != null) {
		while (true) {
		    steps = counter.inc();
		    totalsteps++;
		}
	    }
	} catch (RemoteException ex) {
	    ex.printStackTrace();
	    showStatus("RemoteException during increment.");
	}
    }

    /** Update the display every second, with the current stats
     */
    private void showsteps() {
	while (true) {
	    long firstdate = System.currentTimeMillis();
	    int firststeps = steps;

	    try {
		Thread.sleep(1000);	// sleep for a sec
	    } catch (InterruptedException ex) {
	    }

	    long lastdate = System.currentTimeMillis();
	    int laststeps = steps;
	    long duration = lastdate - firstdate;
	    showStatus(((laststeps - firststeps) * 1000) / (int)duration +
		       " steps/second");
	    argfield.setText(Integer.toString(steps));
	    resultfield.setText(Integer.toString(totalsteps));
	}
    }
}
