/*
 * @(#)GenericServlet.java	1.8 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet;

import java.io.IOException;
import java.util.Enumeration;

/**
 * Generic servlet base class. This is a convenience class so that generic
 * servlets can share common implementation. Servlet writers inherit from
 * this class and just implement the service method.
 *
 * @version     1.6, 01/19/97
 * @author	Pavani Diwanji
 */
public abstract
class GenericServlet implements Servlet {
    private ServletConfig config;

    /**
     * Returns the servlet context. The servlet context lets a servlet
     * access the environment of the server in which it is running.
     */
    public ServletContext getServletContext() {
	return config.getServletContext();
    }

    /**
     * Gets an initialization parameter of the servlet.
     * @param name the parameter name
     */
    public String getInitParameter(String name) {
	return config.getInitParameter(name);
    }

    /**
     * Returns an enumeration of strings representing the initialization
     * parameter names for this servlet.
     */
    public Enumeration getInitParameterNames() {
	return config.getInitParameterNames();
    }

    /**
     * Logs the class name of the servlet and the given message to the
     * servlet log file.
     * @param msg the message string
     */
    public void log(String msg) {
	getServletContext().log(getClass().getName() + ": "+ msg);
    }

    /**
     * Returns a string containing information about the author, version,
     * and copyright of the servlet.
     */
    public String getServletInfo() {
	return null;
    }

    /**
     * Initializes the servlet. This is called automatically by the system
     * when the servlet is first loaded.
     * @param config servlet configuration information
     * @exception ServletException if a servlet exception has occurred
     */
    public void init(ServletConfig config) throws ServletException {
	this.config = config;
	init();
    }

    /**
     * Returns the startup configuration information for this servlet.
     */
    public ServletConfig getServletConfig() {
	return config;
    }

    /**
     * Initializes the servlet. This is called automatically by the system
     * when the servlet is first loaded.
     * @exception ServletException if a servlet exception has occurred
     */
    public void init() throws ServletException {
	log("init");
    }

    /**
     * Services a single request from the client.
     * @param req the servlet request
     * @param req the servlet response
     * @exception ServletException if a servlet exception has occurred
     * @exception IOException if an I/O exception has occurred
     */
    public abstract void service(ServletRequest req, ServletResponse res)
	throws ServletException, IOException;

    /**
     * Destroys the servlet and cleans up whatever resources are being
     * held.
     */
    public void destroy() {
	log("destroy");
    }

}
