/*
 * @(#)Servlet.java	1.16 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet;

import java.io.IOException;

/**
 * Generic servlet interface.
 *
 * @version     1.16, 03/03/97
 * @author      David Connelly
 * @author	Pavani Diwanji
 */
public
interface Servlet {
    /**
     * Initializes the servlet. This is called automatically by the system
     * when the servlet is first loaded.
     * @param config servlet configuration information
     * @exception ServletException if a servlet exception has occurred
     */
    public void init(ServletConfig config) throws ServletException;

    /**
     * Returns the configuration for this servlet.
     */
    public ServletConfig getServletConfig();

    /**
     * Services a single request from the client.
     * @param req the servlet request
     * @param res the servlet response
     * @exception ServletException if a servlet exception has occurred
     * @exception IOException if an I/O exception has occurred
     */
    public void service(ServletRequest req, ServletResponse res)
	throws ServletException, IOException;

    /**
     * Returns a string containing information about the author, version,
     * and copyright of the servlet.
     */
    public String getServletInfo();

    /**
     * Destroys the servlet and cleans up whatever resources are being
     * held.
     */
    public void destroy();
}
