/*
 * @(#)ServletContext.java	1.12 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet;

import java.io.IOException;
import java.util.Enumeration;

/**
 * An interface for defining a servlet's environment. It is used by
 * a servlet to obtain information about the environment (i.e. Web
 * server) in which it is running.
 *
 * @version     1.12, 03/03/97
 * @author      David Connelly
 * @author	Pavani Diwanji
 */
public
interface ServletContext {
    /**
     * Returns the servlet for the specified name.
     * @param name the name of the servlet
     * @return the Servlet, or null if not found
     * @exception ServletException if the servlet could not be initialized
     */
    public Servlet getServlet(String name) throws ServletException;

    /**
     * Enumerates the servlets in this context (server). Only servlets
     * that are accessible will be returned. The enumeration always
     * includes the servlet itself.
     */
    public Enumeration getServlets();
    
    /**
     * Writes a message to the servlet log file.
     * @param msg the message to be written
     */
    public void log(String msg);

    /**
     * Applies alias rules to the specified virtual path and returns the
     * corresponding real path. Returns null if the translation could not
     * be performed.
     * @param path the real path to be translated
     */
    public String getRealPath(String path);

    /**
     * Returns the mime type of the specified file, or null if not known.
     * @param file file name whose mime type is required
     */
    public String getMimeType(String file);

    /**
     * Returns the name and version of the Web server under which the
     * servlet is running. Same as the CGI variable SERVER_SOFTWARE.
     */
    public String getServerInfo();

    /**
     * Returns an attribute of the server given the specified key name.
     * This allows access to additional information about the server not
     * already provided by the other methods in this interface. Attribute
     * names should follow the same convention as package names, and those
     * beginning with 'com.sun.*' are reserved for use by Sun Microsystems.
     * @param name the attribute key name
     * @return the value of the attribute, or null if not defined
     */
    public Object getAttribute(String name);
}
