/*
 * @(#)ServletOutputStream.java	1.6 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet;

import java.io.OutputStream;
import java.io.IOException;

/**
 * An output stream for writing servlet responses.
 *
 * @version	1.6, 03/03/97
 * @author	David Connelly
 */
public abstract
class ServletOutputStream extends OutputStream {
    /**
     * Prints a string.
     * @exception IOException if an I/O error has occurred
     */
    public void print(String s) throws IOException {
	int len = s.length();
	for (int i = 0; i < len; i++) {
	    write(s.charAt(i));
	}
    }

    /**
     * Prints a boolean.
     * @exception IOException if an I/O error has occurred.
     */
    public void print(boolean b) throws IOException {
	print(b ? "true" : "false");
    }

    /*
     * Prints a character.
     * @exception IOException if an I/O error has occurred
     */
    public void print(char c) throws IOException {
	print(String.valueOf(c));
    }

    /**
     * Prints an integer.
     * @exception IOException if an I/O error has occurred
     */  
    public void print(int i) throws IOException {
	print(String.valueOf(i));
    }
 
    /**
     * Prints a long.
     * @exception IOException if an I/O error has occurred
     */
    public void print(long l) throws IOException {
	print(String.valueOf(l));
    }

    /**
     * Prints a float.
     * @exception IOException if an I/O error has occurred
     */
    public void print(float f) throws IOException {
	print(String.valueOf(f));
    }

    /**
     * Prints a double.
     * @exception IOException if an I/O error has occurred
     */
    public void print(double d) throws IOException {
	print(String.valueOf(d));
    }

    /**
     * Prints a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println() throws IOException {
	print("\r\n");
    }

    /**
     * Prints a string followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(String s) throws IOException {
	print(s);
	println();
    }

    /**
     * Prints a boolean followed by a CRLF.
     * @exception IOException if an I/O error has occurred.
     */
    public void println(boolean b) throws IOException {
	print(b);
	println();
    }

    /*
     * Prints a character followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(char c) throws IOException {
	print(c);
	println();
    }

    /**
     * Prints an integer followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(int i) throws IOException {
	print(i);
	println();
    }

    /**  
     * Prints a long followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */  
    public void println(long l) throws IOException {
	print(l);
	println();
    }

    /**
     * Prints a float followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(float f) throws IOException {
	print(f);
	println();
    }

    /**
     * Prints a double followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(double d) throws IOException {
	print(d);
	println();
    }
}
