/*
 * @(#)ServletRequest.java	1.26 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet;

import java.io.IOException;
import java.util.Enumeration;

/*
 * This interface represents a generic servlet request.
 *
 * @version	1.26,03/03/97
 * @author	David Connelly
 */
public
interface ServletRequest {
    /**
     * Returns the size of the request entity data, or -1 if not known.
     * Same as the CGI variable CONTENT_LENGTH.
     */
    public int getContentLength();

    /**
     * Returns the Internet Media Type of the request entity data, or
     * null if not known. Same as the CGI variable CONTENT_TYPE.
     */
    public String getContentType();

    /**
     * Returns the protocol and version of the request as a string of the
     * form &lt;protocol&gt;/&lt;major version&gt;.&lt;minor version&gt.
     * Same as the CGI variable SERVER_PROTOCOL.
     */
    public String getProtocol();

    /**
     * Returns the host name of the server that received the request.
     * Same as the CGI variable SERVER_NAME.
     */
    public String getServerName();

    /**
     * Returns the port number on which this request was received.
     * Same as the CGI variable SERVER_PORT.
     */
    public int getServerPort();

    /**
     * Returns the IP address of the agent that sent the request.
     * Same as the CGI variable REMOTE_ADDR.
     */
    public String getRemoteAddr();

    /**
     * Returns the fully qualified host name of the agent that sent the
     * request. Same as the CGI variable REMOTE_HOST.
     */
    public String getRemoteHost();

    /**
     * Applies alias rules to the specified virtual path and returns the
     * corresponding real path. getRealPath("/") is the document root.
     * It returns null if the translation cannot be performed.
     * @param path the path to be translated
     */  
    String getRealPath(String path);

    /**
     * Returns an input stream for reading the request body.
     */
    public ServletInputStream getInputStream() throws IOException;  

    /**
     * Returns the value of the specified parameter for the request. For
     * example, in an HTTP servlet this would return the value of the
     * specified query string parameter.
     * @param name the parameter name
     */
    public String getParameter(String name);

    /**
     * Returns an enumeration of strings representing the parameter names
     * for this request.
     */
    public Enumeration getParameterNames();

    /**
     * Returns an attribute of the request given the specified key name.
     * This allows access to request information not already provided by
     * the other methods in this interface. Key names beginning with
     * 'COM.sun.*' are reserved.
     * @param name the attribute name
     * @return the value of the attribute, or null if not defined
     */
    public Object getAttribute(String name);
}
