/*
 * @(#)HttpServletResponse.java	1.8 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet.http;

import java.servlet.ServletResponse;
import java.io.IOException;

/**
 * This interface represents an HTTP servlet response. In the servlet's
 * service method, cast the ServletResponse parameter to HttpServletResponse
 * in order to set HTTP protocol specified header information for the
 * response.
 *
 * @version	1.8, 03/03/97
 * @author	David Connelly
 */
public
interface HttpServletResponse extends ServletResponse {
    /**
     * Sets the status code and message for this response.
     * @param sc the status code
     * @param sm the status message
     */
    public void setStatus(int sc, String sm);

    /**
     * Sets the status code and a default message for this response.
     * @param sc the status code
     */
    public void setStatus(int sc);

    /**
     * Sets the value of a header field.
     * @param name the header field name
     * @param value the header field value
     */
    public void setHeader(String name, String value);

    /**
     * Sets the value of an integer header field.
     * @param name the header field name
     * @param value the header field integer value
     */
    public void setIntHeader(String name, int value);

    /**
     * Sets the value of a date header field.
     * @param name the header field name
     * @param value the header field date value
     */
    public void setDateHeader(String name, long date);

    /**
     * Sends an error response to the client using the specified status
     * code and detail message.
     * @param sc the status code
     * @param msg the detail message
     * @exception IOException If an I/O error has occurred.
     */
    public void sendError(int sc, String msg) throws IOException;

    /**
     * Sends an error response to the client using the specified status
     * code and no default message.
     * @param sc the status code
     * @exception IOException If an I/O error has occurred.
     */
    public void sendError(int sc) throws IOException;

    /**
     * Sends a redirect response to the client using the specified redirect
     * location URL.
     * @param location the redirect location URL
     * @exception IOException If an I/O error has occurred.
     */
    public void sendRedirect(String location) throws IOException;

    /*
     * Server status codes.
     */
    public static final int SC_OK = 200;
    public static final int SC_CREATED = 201;
    public static final int SC_ACCEPTED = 202;
    public static final int SC_NO_CONTENT = 204;
    public static final int SC_MOVED_PERMANENTLY = 301;
    public static final int SC_MOVED_TEMPORARILY = 302;
    public static final int SC_NOT_MODIFIED = 304;
    public static final int SC_BAD_REQUEST = 400;
    public static final int SC_UNAUTHORIZED = 401;
    public static final int SC_FORBIDDEN = 403;
    public static final int SC_NOT_FOUND = 404;
    public static final int SC_INTERNAL_SERVER_ERROR = 500;
    public static final int SC_NOT_IMPLEMENTED = 501;
    public static final int SC_BAD_GATEWAY = 502;
    public static final int SC_SERVICE_UNAVAILABLE = 503;
}
