/*
 * @(#)HttpUtils.java	1.4 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package java.servlet.http;

import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * A collection of static utility methods useful to HTTP servlets.
 */
public class HttpUtils {

    /**
     * Parses a query string and builds a hashtable of key, value pairs.
     * Multiple parameters of the same name show up in the value as
     * a comma-delimited list.
     * @exception IllegalArgumentException If the query string is invalid.
     */
    static public Hashtable parseQueryString(String s) {

	if (s == null) {
	    throw new IllegalArgumentException();
	}
	Hashtable ht = new Hashtable();
	StringBuffer sb = new StringBuffer();
	StringTokenizer st = new StringTokenizer(s, "&");
	while (st.hasMoreTokens()) {
	    String pair = (String)st.nextToken();
	    int pos = pair.indexOf('=');
	    if (pos == -1) {
		throw new IllegalArgumentException();
	    }
	    String key = parseName(pair.substring(0, pos), sb);
	    String val = parseName(pair.substring(pos+1, pair.length()), sb);
	    if (ht.containsKey(key)) {
		val = (String) ht.get(key) + "," + val;
	    }
	    ht.put(key, val);
	}
    return ht;
    }

    /*
     * Parse a name in the query string.
     */
    static private String parseName(String s, StringBuffer sb) {
	sb.setLength(0);
	for (int i = 0; i < s.length(); i++) {
	    char c = s.charAt(i);
	    switch (c) {
	      case '+':
		sb.append(' ');
		break;
	      case '%':
		try {
		    sb.append((char) Integer.parseInt(s.substring(i+1, i+3), 
                        16));
		} catch (NumberFormatException e) {
		    throw new IllegalArgumentException();
		}
		i += 2;
		break;
	      default:
		sb.append(c);
		break;
	    }
	}
	return sb.toString();
    }

}
