/*
 * @(#)NSRunner.java	1.6 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package sun.servlet.netscape;

import java.servlet.*;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import sun.servlet.ServletLoader;
import sun.servlet.util.Pool;
import netscape.server.applet.HttpApplet;
import netscape.server.applet.Misconfiguration;
import netscape.net.URLConnection;

/**
 * Netscape server application for running servlets.
 *
 * @version	1.6, 03/03/97
 * @author	David Connelly
 */
public
class NSRunner extends HttpApplet implements ServletContext, ServletConfig {
    /*
     * The class loaders for all loaded servlets.
     */
    private static Hashtable loaders = new Hashtable();

    /*
     * The pool of available request objects.
     */
    private static Pool reqPool = new Pool("sun.servlet.netscape.NSRequest");

    /*
     * The pool of available response objects.
     */
    private static Pool resPool = new Pool("sun.servlet.netscape.NSResponse");

    /*
     * The class loader for the servlet being invoked.
     */
    private ServletLoader loader;

    /**
     * Constructs a new server application for invoking servlets.
     */
    public NSRunner() {
    }

    /**
     * Runs the specified servlet.
     */
    public void run() throws Exception {
	// Get servlet directory
	String vpath = getConfigProperty("vpath");
	if (vpath == null) {
	    vpath = "/servlet";
	}
	String dir = translateURI(vpath);
	if (dir == null) {
	    throw new Misconfiguration("no mapping for " + vpath);
	}
	// Get class loader for directory, and create new loader if not found.
	loader = (ServletLoader)loaders.get(dir);
	if (loader == null) {
	    loader = new ServletLoader(dir);
	    loaders.put(dir, loader);
	}
	// Get name and path info for servlet being invoked
	String uri = getRequestProperty("uri");
	if (!uri.startsWith(vpath)) {
	    throw new Misconfiguration("uri must start with " + vpath);
	}
	int i = vpath.length() + 1;
	if (i > uri.length()) {
	    throw new Misconfiguration("no servlet specified");
	}
	int j = uri.indexOf("/", i);
	NSRequest req = (NSRequest)reqPool.alloc();
	req.init(this);
	String name;
	if (j == -1) {
	    name = uri.substring(i);
	    req.pathInfo = null;
	    req.servletPath = uri;
	} else {
	    name = uri.substring(i, j);
	    req.pathInfo = uri.substring(j);
	    req.servletPath = uri.substring(0, j);
	}
	// Get selected servlet and dispatch request
	Servlet s = getServlet(name);
	if (s == null) {
	    throw new Misconfiguration("servlet " + name + " not found");
	}
	NSResponse res = (NSResponse)resPool.alloc();
	res.init(this);
	s.service(req, res);
	res.finish();
	// Free up request / response objects after use
	req.reset();
	reqPool.free(req);
	res.reset();
	resPool.free(res);
    }
   
    /**
     * Starts a response for the server application.
     */
    public int startResponse() throws IOException {
	return super.startResponse();
    }

    /**
     * Loads a servlet from the current class loader.
     * @param name the Servlet class name
     * @return the Servlet, or null if not found
     * @exception ServletException if an initialization error has occurred
     */
    public Servlet getServlet(String name) throws ServletException {
	return loader.loadServlet(name, this);
    }

    /**
     * Returns an enumeration of all the currently loaded servlets.
     */
    public Enumeration getServlets() {
	return loader.getServlets();
    }

    /**
     * Returns the translated path for the specified virtual path.
     * @param path the virtual path
     * @return the translated path
     */
    public String getRealPath(String path) {
	return translateURI(path);
    }

    /**
     * Returns the MIME type for the specified file name.
     * @param name the file name
     * @return the MIME type of the file
     */
    public String getMimeType(String name) {
	 String type = URLConnection.guessContentTypeFromName(name);
	 return type != null ? type : "text/plain";
    }

    /**
     * Returns the name of this server. Unfortunately, there is no
     * NSAPI property to query for this so the name is hardcoded here.
     */
    public String getServerInfo() {
	return "Netscape Enterprise/2.x";
    }

    /**
     * Logs a message for a servlet.
     * @param msg the log message
     */
    public void log(String msg) {
        inform(msg);
    }

    /**
     * Returns the servlet context for this server.
     */
    public ServletContext getServletContext() {
	return this;
    }

    /**
     * Returns the value of the specified init parameter. In Netscape's
     * server, initialization parameters are specified in the Service
     * directive of the Object declaration for the servlet. See release
     * notes for more details.
     * @param name the parameter name
     * @return the parameter value
     */
    public String getInitParameter(String name) {
	return getServerProperty(name);
    }

    /**
     * Returns an enumeration of init parameter names. Unfortunately, this
     * is not possible in Netscape's server so we just return null.
     */
    public Enumeration getInitParameterNames() {
	return null;
    }

    /**
     * Returns an attribute of the server for the specified key name.
     * @param name the attribute name
     */
    public Object getAttribute(String name) {
	return null;
    }

    /*
     * Sends exception stack trace to the client for debugging.
     */
    void sendDebug(Exception e) throws IOException {
        if (returnErrorResponse("text/html", SERVER_ERROR)) {
	    java.io.PrintStream out = getOutputStream();
	    out.println("<html>");
	    e.printStackTrace(out);
	    out.println("</html>");
	    out.flush();
	}
    }
}
