package npac.webmail;

import java.net.*;
import java.io.*;
import java.util.*;

public class IMAPClient extends FolderSuperclass {
  public static final String HOST = "postoffice.npac.syr.edu";
  public static final int PORT = 143;

  private String user;
  private String host;
  private int port;
  private String passwd;

  private Socket soc;
  private DataInputStream in;
  private PrintStream out;

  //  private int numMessages;
  private int numMessages;

  private static final int NONE = 0, STAR = 1, FETCH = 2, NUM = 3;
  private static final int FLAGS = 21, INTERNALDATE = 22, ENVELOPE = 23, RFC822_SIZE = 24, BODY = 25, BODYCONTENTS = 26, BODYCONTENTSDONE = 261;
  private static final int ENV_DATE = 2301, ENV_SUBJ = 2302, ENV_FROM = 2303,
  			   ENV_SENDER = 2304, ENV_REPLYTO = 2305, ENV_TO = 2306,
			   ENV_CC = 2307, ENV_BCC = 2308, ENV_INREPLYTO = 2309,
			   ENV_MSGID =2310;

  private void login () {
    out.println ("abcd LOGIN " + user + " " + passwd);
    try {
      for (;;) {
	String readline = in.readLine ();
	if (readline.startsWith("abcd") || readline.startsWith("OK") )
	  break;
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
  }

  public int selectMailbox ( String str ) {
    String line = "", lineOld = "";

    System.out.println ("\n-- Executing: \"abcd SELECT " + str +"\"" );
    out.println ("abcd SELECT " + str );
    try {
      for (;;) {
	lineOld = line;
	line = in.readLine ();
	if ( line.startsWith ( "* " ) && line.endsWith ( "EXISTS" ) )
	  numMessages = Integer.parseInt( line.substring ( "* ".length(), line.lastIndexOf(" EXISTS") ) );
        else if (line.startsWith("abcd") || line.startsWith("OK") )
	  break;
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
    catch (NumberFormatException e) { System.out.println(e.toString()); }
    return numMessages;
  }

  public IndexFormat index () {
    String line="";
    int state = NONE;

    if (numMessages == -1)
      selectMailbox ( "INBOX" );
    IndexFormat ind = new IndexFormat ( numMessages );
    if (numMessages > 0) {
      try {
	int i = 1;
	out.println ("abcd FETCH " + i + " (FLAGS RFC822.SIZE ENVELOPE)");
	for ( ; ; ) {
	  line = in.readLine ();
	  if ( line.startsWith("* ") ) {
	    IMAP2bisFilter fil = new IMAP2bisFilter ( line );
	    String token = "";
	    
	    while ( (token = fil.filter()) != null ) {
	      token.trim();
	      switch (state) {
	      case NONE: 
		if (token.equals("*"))
		  state = STAR;
		else
		  System.out.println ("Error. Expected * got ["+token+"]");
		break;
	      case STAR: 
		if (token.equals(String.valueOf(i)))
		  state = NUM;
		else
		  System.out.println ("Error. Expected "+i+" got ["+token+"]");
		break;
	      case NUM:
		if (token.equals("FETCH"))
		  state = FETCH;
		else
		  System.out.println ("Error. Expected FETCH got ["+token+"]");
		break;
	      case FETCH:
		IMAP2bisFilter fetchFil = new IMAP2bisFilter ( token );
		String fetchTok = "";
		
		while ( (fetchTok = fetchFil.filter ()) != null ) {
		  fetchTok.trim();
		  switch (state) {
		  case FETCH:
		    if (fetchTok.equals("FLAGS"))
		      state = FLAGS;
		    else if (fetchTok.equals("INTERNALDATE"))
		      state = INTERNALDATE;
		    else if (fetchTok.equals("RFC822.SIZE"))
		      state = RFC822_SIZE;
		    else if (fetchTok.equals("ENVELOPE"))
		      state = ENVELOPE;
		    break;
		  case FLAGS:
		    IMAP2bisFilter flagsFil = new IMAP2bisFilter ( fetchTok );
		    String flagsTok = "";
		    
		    while ( (flagsTok = flagsFil.filter ()) != null ) {
		      flagsTok.trim();
		      if (flagsTok.equals("\\Seen")) {
			ind.setIsSeen (i-1, true);
                        ind.decrNumUnread ();
		      }
		      else if (flagsTok.equals("\\Flagged"))
			ind.setIsFlagged (i-1, true);
		      else if (flagsTok.equals("\\Deleted")) {
			ind.setIsDeleted (i-1, true);
                        ind.incrNumDeleted ();
		      }
		      else if (flagsTok.equals("\\Answered")) {
			ind.setIsAnswered (i-1, true);
                        ind.incrNumAnswered ();
		      }
		      else 
			/* error */ ;
		    }
		    state = FETCH;
		    break;
		  case INTERNALDATE:
		    state = FETCH;
		    break;
		  case RFC822_SIZE:
		    ind.setMessageSize (i-1, fetchTok);
		    state = FETCH;
		    break;
		  case ENVELOPE:
		    IMAP2bisFilter envFil = new IMAP2bisFilter ( fetchTok );
		    String envTok = "";
		    
		    while ( (envTok = envFil.filter()) != null ) {
		      envTok.trim();
		      switch (state) {
		      case ENVELOPE:
			ind.setDate( i-1, envTok.equals("NIL") ? "" : envTok );
			state = ENV_DATE;
			break;
		      case ENV_DATE:
			ind.setSubject(i-1,envTok.equals("NIL") ? "" : envTok);
			state = ENV_SUBJ;
			break;
		      case ENV_SUBJ:
		        if (!envTok.equals("NIL")) {
			  IMAP2bisFilter fromFil = new IMAP2bisFilter (envTok);
                          //  Assuming only ONE "from"
			  String fromTok = fromFil.filter ();
		          ind.setFrom ( i-1, new AddressFormat ( fromTok ) );
		        }
			else 
			  ind.setFrom ( i-1, new AddressFormat () );
			state = ENV_FROM;
			break;
		      case ENV_FROM:
			state = ENV_SENDER;
			break;
		      case ENV_SENDER:
			state = ENV_REPLYTO;
			break;
		      case ENV_REPLYTO:
			state = ENV_TO;
			break;
		      case ENV_TO:
			state = ENV_CC;
			break;
		      case ENV_CC:
			state = ENV_BCC;
			break;
		      case ENV_BCC:
			state = ENV_INREPLYTO;
			break;
		      case ENV_INREPLYTO:
			state = ENV_MSGID;
			break;
		      case ENV_MSGID:
			state = FETCH;
			break;
		      }
		    }
		    state = FETCH;
		    break;
		  }
		} /* while ( (fetchTok = fetchFil.filter ()) != null ) */
	      }
	    } /* while ( (token = fil.filter()) != null ) */
	  } /* if line.startsWith ("*") */
	  else if (line.equals ("abcd OK FETCH completed")) {
	    if (++i > numMessages)
	      break;
	    out.println ("abcd FETCH " + i + " (FLAGS RFC822.SIZE ENVELOPE)");
	  }
	} /* for ( ;; ) */
      }
      catch (IOException e) { System.out.println(e.toString()); }
    }
    return ind;
  }

  public MailFormat body (int i) {
    String line="";
    String token="";
    int preambleLength = -1;
    int numBody = 0;
    int state = NONE;
    int mesgLenSoFar = 0;
    int mesgLen = 0;
    Vector v = new Vector ();
    MailFormat mf = new MailFormat ();

    try {
      System.out.println ( "abcd FETCH " + i + " BODY");
      out.println ( "abcd FETCH " + i + " BODY");
      for (;;) {
	line = in.readLine();
	//System.out.println ( "Line >>" + line );
	if (line.startsWith("* ")) {
	  IMAP2bisFilter fil = new IMAP2bisFilter ( line );
	  while ( (token = fil.filter()) != null ) {
	    token.trim();
	    switch (state) {
	     case NONE: 
	      if (token.equals("*"))
		state = STAR;
	      else 
		System.out.println ("Error. Expected * got ["+token+"]");
	      break;
	     case STAR: 
	      if (token.equals(String.valueOf(i)))
		state = NUM;
	      else 
		System.out.println ("Error. Expected "+i+" got ["+token+"]");
	      break;
	     case NUM:
	      if (token.equals("FETCH"))
		state = FETCH;
	      else 
		System.out.println ("Error. Expected FETCH got ["+token+"]");
	      break;
	     case FETCH:
	      IMAP2bisFilter fetchFil = new IMAP2bisFilter ( token );
	      String fetchTok = "";
	      
	      while ( (fetchTok = fetchFil.filter ()) != null ) {
		fetchTok.trim();
		switch ( state ) {
		 case FETCH:
		  if (fetchTok.startsWith("BODY[")) {
		    preambleLength = token.length();
		    state = BODYCONTENTS;
		  }
		  else if (fetchTok.equals("FLAGS"))
		    state = FLAGS;
		  else if (fetchTok.equals("BODY"))
		    state = BODY;
		  else if (fetchTok.startsWith("ENVELOPE"))
		    state = ENVELOPE;
		  else 
		    System.out.println ("Error. Expected BODY/ BODY[<number>]/ ENVELOPE got ["+token+"]");
		  break;
		 case FLAGS:
		  IMAP2bisFilter flagsFil = new IMAP2bisFilter ( fetchTok );
		  String flagsTok = "";
		  while ( (flagsTok = flagsFil.filter ()) != null ) {
		    flagsTok.trim();
		    if (flagsTok.equals("\\Seen"))
		      mf.setIsSeen ( true );
		    else if (flagsTok.equals("\\Flagged"))
		      mf.setIsFlagged ( true );
		    else if (flagsTok.equals("\\Deleted"))
		      mf.setIsDeleted ( true );
		    else if (flagsTok.equals("\\Answered"))
		      mf.setIsAnswered ( true );
		    else 
		      /* error */ ;
		  }
		  state = FETCH;
		  break;
		 case BODY:
		  IMAP2bisFilter bodyFil = new IMAP2bisFilter ( fetchTok );
		  String bodyTok = "";
		  
		  while ( (bodyTok = bodyFil.filter()) != null ) {
		    bodyTok.trim();
		    numBody++;
		  }
		  System.out.println ( "abcd FETCH " + i + " (FLAGS ENVELOPE BODY[1])" );
		  out.println ( "abcd FETCH " + i + " (FLAGS ENVELOPE BODY[1])" );
		  state = NONE;
		  break;
		case ENVELOPE:
		  IMAP2bisFilter envFil = new IMAP2bisFilter ( fetchTok );
		  String envTok = "";
		  
		  while ( (envTok = envFil.filter()) != null ) {
		    envTok.trim();
		    switch (state) {
		    case ENVELOPE:
		      mf.setDate ( envTok.equals("NIL") ? "" : envTok );
		      state = ENV_DATE;
		      break;
		    case ENV_DATE:
		      mf.setSubject ( envTok.equals("NIL") ? "" : envTok );
		      state = ENV_SUBJ;
		      break;
		    case ENV_SUBJ:
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter fromFil = new IMAP2bisFilter (envTok);
			//  Assuming only ONE "from"
			String fromTok = fromFil.filter ();
			mf.setFrom ( new AddressFormat ( fromTok ) );
		      }
		      else 
			mf.setFrom ( new AddressFormat () );
		      state = ENV_FROM;
		      break;
		    case ENV_FROM:
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter senderFil = new IMAP2bisFilter (envTok);
			//  Assuming only ONE "sender"
			String senderTok = senderFil.filter ();
			mf.setSender ( new AddressFormat(senderTok) );
		      }
		      else 
			mf.setSender ( new AddressFormat () );
		      state = ENV_SENDER;
		      break;
		    case ENV_SENDER:
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter replyToFil = new IMAP2bisFilter(envTok);
			//  Assuming only ONE "replyTo"
			String replyToTok = replyToFil.filter ();
			mf.setReplyTo ( new AddressFormat(replyToTok) );
		      }
		      else 
			mf.setReplyTo ( new AddressFormat () );
		      state = ENV_REPLYTO;
		      break;
		     case ENV_REPLYTO:
		      if (!v.isEmpty()) v.removeAllElements ();
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter toFil = new IMAP2bisFilter( envTok );
			String toTok = "";
			while ( (toTok = toFil.filter ()) != null )
			  v.addElement ( new AddressFormat(toTok) );
		      }
		      mf.setTo ( v );
		      state = ENV_TO;
		      break;
		    case ENV_TO:
		      if (!v.isEmpty()) v.removeAllElements ();
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter ccFil = new IMAP2bisFilter ( envTok );
			String ccTok = "";
			while ( (ccTok = ccFil.filter ()) != null ) {
			  v.addElement ( new AddressFormat ( ccTok ) );
			}
		      }
		      mf.setCc ( v );
		      state = ENV_CC;
		      break;
		    case ENV_CC:
		      if (!v.isEmpty()) v.removeAllElements ();
		      if (!envTok.equals("NIL")) {
			IMAP2bisFilter bccFil = new IMAP2bisFilter ( envTok );
			String bccTok = "";
			while ( (bccTok = bccFil.filter ()) != null )
			  v.addElement ( new AddressFormat ( bccTok ) );
		      }
		      mf.setBcc ( v );
		      state = ENV_BCC;
		      break;
		    case ENV_BCC:
		      mf.setInReplyTo ( envTok );
		      state = ENV_INREPLYTO;
		      break;
		    case ENV_INREPLYTO:
		      mf.setMsgId ( envTok.equals("NIL") ? "" : envTok );
		      state = ENV_MSGID;
		      break;
		    case ENV_MSGID:
		      state = FETCH;
		      break;
		    }
		  }
		  state = FETCH;
		  break;
		case BODYCONTENTS:
		  preambleLength += 1 + fetchTok.length ();
		  mesgLen = Integer.parseInt( fetchTok.substring(1,fetchTok.length()-1) );
		  if (!v.isEmpty()) v.removeAllElements ();
		  break;
		}
	      }
	    }
	  }
	}
	else if (state == BODYCONTENTS) {
	  v.addElement ( line );
	  mesgLenSoFar += line.length() + 2; // "+2" for CR + LF
	  if (mesgLenSoFar == mesgLen )
	    state = BODYCONTENTSDONE;
	}
	else if (state == BODYCONTENTSDONE) {
	  mf.setMessageBody ( v );
	  if ( line.equals ("abcd OK FETCH completed") )
	    break;
	}
      }
      out.println ("abcd STORE " + i + " +" + "FLAGS (\\Seen)");
      for (;;) {
	line = in.readLine ();
	if (line.equals("abcd OK +FLAGS completed"))
	  break;
	else if (line.startsWith("abcd NO") || line.startsWith("abcd BAD")) {
	  System.out.println ("Bad command");
	  break;
	}
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
    return mf;
  }

  // For DELETE set del = true and for UNDELETE set del = false
  public boolean delete ( int i, boolean del ) {
    out.println ("abcd STORE " + i + " " + (del ? "+" : "-") + "FLAGS (\\Deleted)");
    try {
      for (;;) {
	String line = in.readLine ();
	if (line.equals("abcd OK " + (del ? "+" : "-") + "FLAGS completed"))
	  return true;
	else if (line.startsWith("abcd NO") || line.startsWith("abcd BAD")) {
	  System.out.println ("Bad command");
	  return false;
	}
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
    return false;
  }

  public void expunge () throws IOException {
    out.println ("abcd EXPUNGE");
    try {
      for (;;) {
	String readline = in.readLine ();
	System.out.println("readline in IMPAC expunge = " + readline);
 	if (readline.startsWith("abcd OK Expunged"))
	  break;
	else if (readline.startsWith("abcd NO") || readline.startsWith("abcd BAD")) {
	  System.out.println ("Bad command");
	  break;
	}
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
   
    //return index();
  }

  public void fetch () {
    out.println ("abcd FETCH 2:4 (FLAGS BODY[HEADER.FIELDS (DATE FROM)])");
    try {
      for (;;) {
	String readline = in.readLine ();
	if (readline.startsWith("abcd") || readline.startsWith("OK") )
	  break;
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
  }

  private void saveIntoFile ( int c, File file ) throws NoSuchFolderException {
    RandomAccessFile localRAF = null;
    try {
      localRAF = new RandomAccessFile ( file, "rw" );
      localRAF.seek ( localRAF.length() );
      Date d = new Date();
      MailFormat mf = body(c);
      mf.setIsDeleted ( false );
      String x = "From "+user+"@"+host+" "+d.toLocaleString()+"\n"+mf;
      localRAF.writeBytes ( x );
      localRAF.close();
    } catch (IOException ioe) {
      throw ( new NoSuchFolderException () );
    }
  }

  public void save ( int c, File folder ) throws NoSuchFolderException {
    if (!folder.exists())
      throw ( new NoSuchFolderException () );
    saveIntoFile(c, folder);
  }

  public void save ( int c, String fullFilePath ) throws NoSuchFolderException { 
    File f = new File (fullFilePath); 
    if (!f.exists())
      throw ( new NoSuchFolderException () );
    saveIntoFile (c, f);
  }

  public void save ( int c, String directory, String filename ) throws NoSuchFolderException { 
    File f = new File (directory, filename);
    if (!f.exists())
      throw ( new NoSuchFolderException () );
    saveIntoFile (c, f);
  }

  private void list () {
    out.println ("abcd LIST \"\" INBOX");
    try {
      for (;;) {
	String readline = in.readLine ();
	if (readline.startsWith("abcd") || readline.startsWith("OK") )
	  break;
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
  }

  private String command ( String str ) {
    String line="", lineOld="";

    System.out.println ("\n-- Executing: \"abcd " + str +"\"" );
    out.println ("abcd " + str );
    try {
      for (;;) {
	lineOld = line;
	line = in.readLine ();
	//System.out.println (line);
	if (line.startsWith("abcd") || line.startsWith("OK") )
	  break;
      }
    }
    catch (IOException e) { System.out.println(e.toString()); }
    return lineOld;
  }

  public IMAPClient ( String u, String p, String h, int t) {
    user = u;
    passwd = p;
    host = h;
    port = t;
    numMessages = -1;
    try {
      soc = new Socket ( host, PORT );
      in = new DataInputStream(soc.getInputStream());
      out = new PrintStream ( soc.getOutputStream() );
    }
    catch (IOException e) { System.out.println(e.toString()); }
    login ();
    selectMailbox ( "INBOX" );
  }

  public IMAPClient ( String u, String p ) {
    user = u;
    passwd = p;
    host = HOST;
    port = PORT;
    numMessages = -1;
    try {
      soc = new Socket ( host, PORT );
      in = new DataInputStream(soc.getInputStream());
      out = new PrintStream ( soc.getOutputStream() );
    }
    catch (IOException e) { System.out.println(e.toString()); }
    login ();
    selectMailbox ( "INBOX" );
  }
}
