package webmail.imap;

/**
 * A class for parsing the string returned by a IMAP (2 bis) Server.
 * For example:
 * <pre>
 *    IMAP2bisFilter fil = new IMAP2bisFilter ( stringReturnedByIMAPServer );
 *    while ( (String token = fil.filter()) != null )
 *        doSomeProcessing ( token );
 * </pre>
 *
 * @version        1.0 11 Oct 1996
 * @author         Mohan Gummalam and Sangeeta Aggarwal
 */
public class IMAP2bisFilter 
{
  private String localStr = "";
  private int currBeg = -1;
  private int beg = -1;
  private int end = -1;
  private int size = 0;
  private boolean sizeMentioned = false;

  /**
   * @param inputString      The string (from IMAP Server) to parse
   */
  public IMAP2bisFilter ( String inputString ) {
    localStr = inputString;
    currBeg = 0;
    beg = 0;
    end = localStr.length();
  }

  /**
   * Return a boolean.  
   * @return           <tt>true<\tt> if the size of the field was returned 
   *                   instead of the field itself,  <tt>false<\tt> otherwise.
   */
  public boolean getSizeMentioned () { return sizeMentioned; }

  /**
   * Return an int.
   * @return           The size.
   */
  public boolean getSize () { return size; }

  /**
   * Return a token (String).  The token is the string enclosed in "", () or {}
   * @return           The token.  If "", check if <tt>getSizeMentioned()<\tt>
   *                   returns <tt>true<\tt>.  If <tt>true<\tt>, use
   *                   <tt>getSize()</tt> to determine number of bytes to be
   *                   fetched.
   */
  public String filter () {
    String returnStr = "";

    sizeMentioned = false;
    size = 0;
    if ( currBeg >= end )
      return null;
    while ( currBeg<end && (localStr.charAt(currBeg) == ' ') )
      currBeg++;
    if (localStr.substring(currBeg).startsWith("NIL")) {
      currBeg += "NIL".length();
      return "";
    }
    else if ( localStr.charAt(currBeg) == '{' ) {
      for ( int i = currBeg+1; i<end && (localStr.charAt(i) != '}'); i++ )
	;
      returnStr = localStr.substring ( currBeg+1, i );
      currBeg = i+1;
      sizeMentioned = true;
      size = Integer.parseInt ( returnStr );
      return "";
    }
    else if ( localStr.charAt(currBeg) == '"' ) {
      for ( int i = currBeg+1; i<end && (localStr.charAt(i) != '"'); i++ )
	;
      returnStr = localStr.substring ( currBeg+1, i );
      currBeg = i+1;
      return returnStr;
    }
    else if ( localStr.charAt(currBeg) == '(' ) {
      int i = currBeg+1;
      int level = 0;
      while ( i<end && ((localStr.charAt(i) != ')') || (level != 0)) ) {
	if (localStr.charAt(i) == '(')
	  level++;
	if (localStr.charAt(i) == ')')
	  level--;
	i++;
      }
      returnStr = localStr.substring ( currBeg+1, i );
      currBeg = i+1;
      return returnStr;      
    }
    else {
      int i = currBeg;
      while ( i<end && (localStr.charAt(i) != ' ') )
	i++;
      returnStr = localStr.substring ( currBeg, i );
      currBeg = i+1;
      return returnStr;      
    }
  }
}

